@@ -1,4 +1,4 @@
-# $Id: Gcc.pm,v 1.39 2014/01/15 21:32:38 pfeiffer Exp $
+# $Id: Gcc.pm,v 1.41 2016/09/06 19:59:45 pfeiffer Exp $
 
 =head1 NAME
 
@@ -85,9 +85,8 @@ sub xparse_command {
 
   my @prefiles;
   my @files;
-  my $idash; # already saw -I-
+  my $idash; # saw -I-
   my @incdirs;
-  my @inc1dirs; # specified *before* -I-
   my @idirafter;
   my $iprefix = '';
   my @libs;
@@ -125,14 +124,15 @@ sub xparse_command {
       $_ ||= shift @words;
       if( $_ eq '-' ) {
 	$idash = 1;
+	$scanner->add_include_dir( user => $_ )
+	  for splice @incdirs;
       } else {
-	push @{$idash ? \@incdirs : \@inc1dirs}, $_;
+	push @incdirs, $_;
       }
     } elsif( s/^i(?:quot(e)|syste(m)|dirafte(r)|(nclude|macros)|prefi(x)|withprefix(before)?)// ) {
       my $val = $_ || shift @words; # yes, value can be glued :-{
       if( $1 || $2 ) {		# -iquote, -isystem
-	$scanner->add_include_dir( user => $val );
-	$scanner->add_include_dir( sys => $val ) if $2;
+	$scanner->add_include_dir( $2 ? 'sys' : 'user', $val );
       } elsif( $3 ) { 		# -idirafter
 	push @idirafter, $val;
       } elsif( $4 ) {		# -include or -imacros
@@ -140,7 +140,7 @@ sub xparse_command {
       } elsif( $5 ) { 		# -iprefix
 	$iprefix = $val;
       } elsif( $6 ) { 		# -iwithprefixbefore
-	push @{$idash ? \@incdirs : \@inc1dirs}, $iprefix . $val;
+	push @incdirs, $iprefix . $val;
       } else {			# -iwithprefix
 	push @idirafter, $iprefix . $val;
       }
@@ -174,45 +174,34 @@ sub xparse_command {
 
   $self->set_default_signature_method( $leave_comments );
 
-  unless( $idash ) {
-    unshift @incdirs, splice @inc1dirs;
-    $scanner->add_include_dir( user => undef );
-  }
+  $scanner->add_include_dir( user => undef ) unless $idash;
 
   $self->tags( $no_link ? 0 : $static ? 1 : 2 );
 
-  push @incdirs, Mpp::Text::split_path 0, 0, $setenv->{CPATH}
-    if $setenv->{CPATH} && ! exists $self->{xNO_GCC};
-  $scanner->add_include_dir( user => $_ )
-    for @inc1dirs, @incdirs;
+  $scanner->add_include_dir( user => 'in CPATH' ) unless exists $self->{xNO_GCC};
+  $scanner->add_include_dir( user => 'in :sys' );
   $scanner->add_include_dir( sys => $_ )
     for @incdirs;
   if( $nostdinc ) {
     $scanner->should_find( 'sys' );
   } else {
     require Mpp::Subs;
-    for my $dir ( @Mpp::Subs::system_include_dirs ) {
-      $scanner->add_include_dir( $_, $dir )
-	for qw(user sys);
-    }
-  }
-  for my $dir ( @idirafter ) {
-    $scanner->add_include_dir( $_, absolute_filename file_info $dir, $self->{RULE}{MAKEFILE}{CWD} )
-      for qw(user sys);
+    $scanner->add_include_dir( sys => $_ )
+      for @Mpp::Subs::system_include_dirs;
   }
+  $scanner->add_include_dir( sys => absolute_filename file_info $_, $self->{RULE}{MAKEFILE}{CWD} )
+    for @idirafter;
   my $context = $scanner->get_context if @files > 1;
+  my $var;
   for( @files ) {
     my $file_end = case_sensitive_filenames ? lc substr $_, -4 : substr $_, -4;
     my $cplusplus = $file_end =~ /\.(?:c(?:c|xx|pp|\+\+)|C)$/;
     $scanner->reset( $context, $cplusplus ) if $context;
-    my $var = exists $self->{xNO_GCC} ?
-      Mpp::is_windows && $cmd =~ /\bcl(?:.cl)?$/i && 'INCLUDE' :
-      $cplusplus ? 'CPLUS_INCLUDE_PATH' : 'C_INCLUDE_PATH';
-    if( $var && $setenv->{$var} ) {
-      for( Mpp::Text::split_path 0, 0, $setenv->{$var}, Mpp::is_windows < 0 && $var eq 'INCLUDE' ) {
-	$scanner->add_include_dir( user => $_ );
-	$scanner->add_include_dir( sys => $_ );
-      }
+    unless( $var ) {
+      $var = exists $self->{xNO_GCC} ?
+	Mpp::is_windows && $cmd =~ /\bcl(?:.cl)?$/i && 'in ;INCLUDE' :
+	$cplusplus ? 'in CPLUS_INCLUDE_PATH' : 'in C_INCLUDE_PATH';
+      $scanner->add_include_dir( sys => $var, 1 ) if $var;
     }
 
     $self->xset_preproc_vars( $cmd, \@cpp_opts, $cplusplus ) if $conditional;
@@ -227,12 +216,9 @@ sub xparse_command {
     $scanner->add_include_dir( lib => $_ )
       for @Mpp::Subs::system_lib_dirs;
     my $var = exists $self->{xNO_GCC} ?
-      Mpp::is_windows && $cmd =~ /\bcl(?:.cl)?$/i && 'LIB' :
-      'LIBRARY_PATH';
-    if( $var && $setenv->{$var} ) {
-      $scanner->add_include_dir( lib => $_ )
-	for Mpp::Text::split_path 0, 0, $setenv->{$var}, Mpp::is_windows < 0 && $var eq 'LIB';
-    }
+      Mpp::is_windows && $cmd =~ /\bcl(?:.cl)?$/i && 'in ;LIB' :
+      'in LIBRARY_PATH';
+    $scanner->add_include_dir( lib => $var ) if $var;
 
     $self->add_simple_dependency( $_ )
       for @obj_files;
