@@ -1,4 +1,4 @@
-# $Id: FileOpt.pm,v 1.123 2014/03/19 21:56:01 pfeiffer Exp $
+# $Id: FileOpt.pm,v 1.125 2014/08/03 21:10:36 pfeiffer Exp $
 
 =head1 NAME
 
@@ -72,7 +72,9 @@ If $no_last_chance is set, then don't consider last chance rules or autoloads.
 sub get_rule {
   return undef if &dont_build;
   my $mdir = $_[0]{'..'};
-  Mpp::Makefile::implicitly_load($mdir); # Make sure we've loaded a makefile for this directory.
+  exists $mdir->{MAKEINFO} or
+    Mpp::Makefile::implicitly_load( $mdir ) # Make sure we've loaded a makefile for this directory.
+    if $Mpp::implicitly_load_makefiles;
   # If we know the rule now, then return it.  Otherwise, try to find a "backwards inference" rule.
   return $_[0]{RULE} if exists $_[0]{RULE};
   if( $n_last_chance_rules && !$_[1] ) {
@@ -138,13 +140,16 @@ exists_or_can_be_built) are cached in EXISTS_OR_CAN_BE_BUILT.  But, since this
 function used to get called an obscene number of times, they don't themselves
 check the cache, instead providing it to its potential caller.
 
+There can be 3 values for phony: 0 -- no phony targets; 1 -- only phony
+targets; undef -- don't care.
+
 =cut
 
 my %warned_stale;
 sub exists_or_can_be_built_norecurse {
   my ($finfo, $phony, $stale) = @_;
-  return exists $finfo->{xPHONY} && $phony ? $finfo : 0
-    if $phony || exists $finfo->{xPHONY};
+  return exists $finfo->{xPHONY} && $finfo if $phony;
+  return $finfo if !defined $phony and exists $finfo->{xPHONY};
 				# Never return phony targets unless requested.
 
   # lstat and stat calls over NFS take a long time, so if we do the lstat and
@@ -178,7 +183,7 @@ sub exists_or_can_be_built_norecurse {
   undef;
 }
 sub exists_or_can_be_built {
-  # If $phony is set, then return only phony targets, which are otherwise ignored.
+  # If $phony is 0, return no phony targets, if 1, only phony targets, if undef any targets.
   # If $stale is set, then return stale generated files too, even if
   # $Mpp::rm_stale_files is set.
   # If $no_last_chance is set, then don't return generated files if the only
@@ -521,8 +526,7 @@ sub set_rule {
 		if $Mpp::log_level;
 	      return if $cmp == 1;
 	    } else {
-	      warn 'rule `', $rule->source, "' produces ", &absolute_filename,
-		" in two different ways\n"
+	      warn $rule->source, ': two different ways to produce `', &absolute_filename, "'\n"
 		if $rule->source eq $oldrule->source;
 	    }
 	  } else {
@@ -534,11 +538,11 @@ sub set_rule {
 	  Mpp::log RULE_IGN_PATTERN => $oldrule
 	    if $Mpp::log_level;
 	} else {
-	  warn 'conflicting rules `', $rule->source, "' and `", $oldrule->source, "' for target ",
-	    &absolute_filename, "\n"
-	      unless exists($rule->{xMULTIPLE_RULES_OK}) &&
-		exists($oldrule->{xMULTIPLE_RULES_OK}) &&
-		$rule->{COMMAND_STRING} eq $oldrule->{COMMAND_STRING};
+	  warn( $rule->source, ": conflicting rule for target `", &absolute_filename, "'\n" ),
+	  warn( $oldrule->source, ": info: was the previous rule\n" )
+	    unless exists $rule->{xMULTIPLE_RULES_OK} &&
+	      exists $oldrule->{xMULTIPLE_RULES_OK} &&
+	      $rule->{COMMAND_STRING} eq $oldrule->{COMMAND_STRING};
 	  # It's not safe to suppress this warning solely because the
 	  # command string is the same, because it might expand differently
 	  # in different makefiles.  But if the rules are marked to allow
@@ -560,8 +564,7 @@ sub set_rule {
     if Mpp::is_windows && $rule_is_builtin && delete $finfo->{_IS_EXE_PHONY_};
 
   if( exists $finfo->{BUILD_HANDLE} && UNIVERSAL::isa $finfo->{RULE}, 'Mpp::Rule' ) {
-    warn 'I became aware of the rule `', $rule->source,
-      "' for target ", &absolute_filename, " after I had already tried to build it\n"
+    warn $rule->source, ': rule discovered for target ', &absolute_filename, " after I had already tried to build it\n"
       unless $rule_is_builtin || exists $rule->{xMULTIPLE_RULES_OK} || UNIVERSAL::isa $rule, 'Mpp::DefaultRule';
   }
 
