@@ -1,7 +1,10 @@
+function getnav() {
+  return document.getElementsByTagName('nav')[0];
+}
 // toggle nav bar left or right
-function lr( cur ) {
+function lr() {
   document.cookie =
-    (cur.parentNode.parentNode.className = cur.parentNode.parentNode.className ? '' : 'right') ?
+    (getnav().className = getnav().className ? '' : 'right') ?
       'makepp_nav=right;expires=01/01/2099 00:00:00' :
       'makepp_nav=;expires=01/01/1970 00:00:00';
 }
@@ -9,20 +12,20 @@ function lr( cur ) {
 // check if nav bar should be right (in Chrome needs --enable-file-cookies)
 function r() {
   if( document.cookie.indexOf('makepp_nav=right') >= 0 ) {
-    document.getElementsByTagName('ul')[0].className = 'right';
+    getnav().className = 'right';
   }
 }
 
 // turn off nav bar
-function nonav( cur ) {
-  cur.parentNode.parentNode.className = 'none';
+function nonav() {
+  getnav().className = 'none';
 }
 
 // roll in or out all parts of nav bar
-function roll( cur, out ) {
+function roll( out ) {
   var now = out ? 'fold' : 'unfold';
   var then = out ? 'unfold' : 'fold';
-  var lis = cur.parentNode.parentNode.childNodes;
+  var lis = getnav().childNodes[0].childNodes;
   for( var i = 1; i < lis.length; i++ )
     if( lis[i].className == now )
       lis[i].className = then;
@@ -31,4 +34,4 @@ function roll( cur, out ) {
 // roll in or out current part of nav bar
 function fold( cur ) {
   cur.className = (cur.className=='fold') ? 'unfold' : 'fold';
-}
\ No newline at end of file
+}
