@@ -5,6 +5,7 @@
 
 package Mpp;
 
+use lib '.';			# May have been removed for security.
 use Config;
 use Cwd;
 use File::Path;
@@ -12,10 +13,10 @@ use File::Path;
 # on some (Windowsish) filesystems rmtree may temporarily fail
 sub slow_rmtree(@) {
   for my $tree ( grep -d, @_ ) {
-    for( 0..9 ) {
-      eval { $@ = ''; local $SIG{__WARN__} = sub { die @_ }; rmtree $tree };
+    for( 0..20 ) {
+      select undef, undef, undef, .1 if $_;
+      eval { local $SIG{__WARN__} = sub { $@ = $_[0] }; rmtree $tree }; # don't die, can have chdir'ed
       -d $tree or last;
-      $_ < 9 and select undef, undef, undef, .1;
     }
     warn $@ if $@;
   }
@@ -41,10 +42,7 @@ my $dot;
 my $verbose;
 my $test;
 my $keep;
-my $name;
-my $perltype;
 my $basedir;
-my $subdir;
 my $dotted;
 our $makepp_path;
 
@@ -65,7 +63,7 @@ BEGIN {
 	last;
       }
     }
-  } else {			# Find it in $PATH:
+  } else {				# Find it in $PATH:
     foreach (split(/:/, $ENV{PATH}), '.') {
       my $dir = $_ || '.';	# Blank path element is .
       if( -x "$dir/$0" ) {
@@ -99,7 +97,7 @@ BEGIN {
 	require Mpp::Text;
 	last;
       }
-      die "Can't locate path to makepp libraries." if $. == 99;
+      die "$0: Can't locate path to makepp libraries." if $. == 99;
     }
   }
 
@@ -125,8 +123,8 @@ BEGIN {
     [qw(d dots), \$dot],
     [qw(k keep), \$keep],
     [qw(m makepp), \$makepp_path, 1],
-    [qw(n name), \$name, 1],
-    [qw(s subdir), \$subdir],
+    [qw(n name), \my $name, 1],
+    [qw(s subdir), \my $subdir],
     [qw(t test), \$test],
     [qw(v verbose), \$verbose],
     [qr/[h?]/, 'help', undef, 0, sub { print <<EOF; exit }] );
@@ -159,9 +157,9 @@ EOF
     }
   }
 
-  $perltype =
+  my $perltype =
     $Config{cf_email} =~ /(Active)(?:Perl|State)/ ? $1 :
-    $Config{ldflags} =~ /(vanilla|strawberry|chocolate)/i ? ucfirst lc $1 :
+    $Config{ldflags} =~ /(vanilla|strawb(?:erry|(?=~))|chocolate)/i ? ucfirst lc $1 :
     '';
 
   printf "%s%sPerl V%vd %dbits - %s %s\n",
@@ -181,9 +179,9 @@ EOF
     $basedir .= "-$perltype" if $perltype;
     $basedir .= "-$name" if $name;
     slow_rmtree $basedir;
-    mkdir $basedir or die "can't mkdir $basedir--$!";
     $basedir .= '/';
   }
+  -d $basedir or c_mkdir( -p => $basedir ) or die "$0: can't mkdir $basedir--$!";
 
   chdir $basedir;
   mkdir 'd';
@@ -193,7 +191,7 @@ EOF
   rmdir 'd';
   unlink 'e' or rmdir 'e';
   eval 'sub no_symlink() {' . ($symlink ? '' : 1) . '}';
-  open my $fh, '>f';		# Use different filename because rmdir may fail on Win
+  open my $fh, '>', 'f';		# Use different filename because rmdir may fail on Win
   close $fh;
   my $link = eval { link 'f', 'g' } &&	# might die somewhere
     ((stat 'f')[1] ?	# Do we have inums?
@@ -285,10 +283,10 @@ sub makepp(@) {
   }
   print "makepp$suffix" . (@_ ? " @_\n" : "\n");
   system_intabort \"makepp$suffix", # "
-    PERL, '-w', exists $file{'makeppextra.pm'} ? '-Mmakeppextra' : (), $makepp_path.$suffix, @_;
+    PERL, '-w', exists $file{'makeppextra.pm'} ? qw(-I. -Mmakeppextra) : (), $makepp_path.$suffix, @_;
   unless( $extra ) {
     for my $file ( <{*/*/*/,*/*/,*/,}.makepp/*.mk> ) {
-      open my( $fh ), $file;
+      open my $fh, '<', $file;
       $file =~ s!\.makepp/(.+)\.mk$!$1!;
       -r $file && !-d _ or next;
       my $binfo = Mpp::File::grok_build_info_file $fh;
@@ -333,7 +331,7 @@ sub un_spar() {
 		-d $name or mkdir $name, 0700 or warn "spar: can't mkdir `$name': $!\n";
 		$mode{$name} = [$atime, $mtime, oct $mode];
 	    } elsif( $kind ne 'L' ) {
-		open F, ">$name" or warn "spar: can't open >`$name': $!\n";
+		open F, '>', $name or warn "spar: can't open >`$name': $!\n";
 		$lines = abs $kind;
 		$nl = ($kind < 0) ? '' : "\n";
 	    }
@@ -400,7 +398,7 @@ sub do_pl($) {
 
 sub n_files(;$$) {
   my( $outf, $code ) = @_;
-  open my $logfh, '.makepp/log' or die ".makepp/log--$!\n";
+  open my $logfh, '<', '.makepp/log' or die ".makepp/log--$!\n";
   seek $logfh, -20, 2 if !$code; # More than enough to find last message.
   open my $outfh, '>', $outf if $outf;
   while( <$logfh> ) {
@@ -483,7 +481,7 @@ foreach $archive (@ARGV) {
       system_intabort $tarcmd and # Extract the tar file.
 	die "$0: can't extract testfile $archive\n";
     } elsif( !$dirtest ) {
-      open DATA, $archive or die "$0: can't open $archive--$!\n";
+      open DATA, '<', $archive or die "$0: can't open $archive--$!\n";
       eval { local $SIG{__WARN__} = sub { die @_ if $_[0] !~ /Failed to set/ }; un_spar };
 				# Alas happens a lot on native Windows.
       die +(is_windows && $@ =~ /symlink .* unimplemented/) ? "skipped s\n" :
@@ -492,7 +490,7 @@ foreach $archive (@ARGV) {
     }
     open STDOUT, '>', $log or die "write $log: $!";
     open STDERR, '>&STDOUT' or die $!;
-    open my $fh, '>>.makepprc';	# Don't let tests be confused by a user's file further up.
+    open my $fh, '>>', '.makepprc';	# Don't let tests be confused by a user's file further up.
     close $fh;
     # check for all special files in one go:
     @file{<{is_relevant.pl,makepp_test_script.pl,makepp_test_script,cleanup_script.pl,makeppextra.pm,hint}*>} = ();
@@ -546,7 +544,7 @@ foreach $archive (@ARGV) {
 #
     if( !defined( my $n_files_updated = n_files )) {
       push @errors, '.makepp/log';
-    } elsif( open my $n_files, 'answers/n_files' ) { # Count of # of files updated?
+    } elsif( open my $n_files, '<', 'answers/n_files' ) { # Count of # of files updated?
       $_ = <$n_files>;
       &$mod_answer( 'n_files', $n_files_updated, $_ ) if $mod_answer;
       $_ eq $n_files_updated
@@ -557,7 +555,7 @@ foreach $archive (@ARGV) {
 # Also search through the log file to make sure there are no Perl messages
 # like "uninitialized value" or something like that.
 #
-    if( open my $logfile, $log ) {
+    if( open my $logfile, '<', $log ) {
       while( <$logfile> ) {
 	# Have to control a few warnings before we can unleash this:
 	#/makepp: warning/
